# Solitude is a interactive story with four endings.
# Copyright (C) 2012 Ryan Martin
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path
import sys
import json

import pyglet
from pyglet.gl import *
from pyglet import sprite
from pyglet import image
from pyglet import resource
from Viewport import FixedResolutionViewport

import GameData
import Interface
from Character import Character
from Entity import Entity
from Stage import Stage

class Wilderness(Stage):
    def __init__(self):
        width, height=self.load_map("wilderness.json")
        Stage.__init__(self, width, height)
        self.background=sprite.Sprite(resource.image("wilderness_background.png"), batch=self.batch, group=self.backgroup)
        self.title=sprite.Sprite(resource.image("title.png"), batch=self.batch, group=self.ogroups[0])
        self.title.image.anchor_x=self.title.width/2
        self.title.image.anchor_y=self.title.height/2
        self.title.x=30*32
        self.title.y=42*32
        #self.foreground=sprite.Sprite(resource.image("wilderness_foreground.png"), batch=self.batch, group=self.foregroup)
        self.cave_foreground=sprite.Sprite(resource.image("wilderness_cave_foreground.png"), batch=self.batch, group=self.ogroups[0])
        self.player.load_walkcycle(resource.image("wilderness_walkcycle.png"))
        self.player.set_idle("down")
        self.player.x=30*32
        self.player.y=42*32
        self.player.food=False
        self.player.water=False
        self.player.sleep=False
        self.set_group(self.player)
        self.update_camera()
        self.icon_images={}
        self.icons={}
        icon_margin=10
        icon_offset=icon_margin
        for icon in ('sleep', 'water', 'food'):
            self.icon_images[icon]=resource.image(icon+".png")
            self.icon_images['no'+icon]=resource.image("no"+icon+".png")
            self.icons[icon]=sprite.Sprite(self.icon_images['no'+icon])
            self.icons[icon].x=icon_margin
            self.icons[icon].y=icon_offset
            icon_offset+=self.icons[icon].height+icon_margin
        tree_image=resource.image("tree.png")
        tree_image.anchor_x=tree_image.width/2
        self.trees=[]
        for i,j in self.tree_locations:
            tree=sprite.Sprite(tree_image, batch=self.batch, group=self.ogroups[1])
            tree.x=i*32+16
            tree.y=j*32
            self.set_group(tree)
            self.trees.append(tree)
        self.fruit_tree=Entity(resource.image("fruit_tree.png"), batch=self.batch, group=self.ogroups[1])
        self.fruit_tree.image.anchor_x=self.fruit_tree.width/2
        self.fruit_tree.x=self.fruit_tree_locations[0][0]*32+16
        self.fruit_tree.y=self.fruit_tree_locations[0][1]*32
        self.fruit_tree.action=lambda text="Eat fruit?", action=self.eat_food: self.show_yes_no_dialog(text, action)
        self.set_group(self.fruit_tree)
        self.add_entity(self.fruit_tree)
        for i,j in self.water_locations:
            self.set_collision(i*32, j*32)
            self.set_action(i*32, j*32, lambda text="Drink water?", action=self.drink_water: self.show_yes_no_dialog(text, action))
        for i,j in self.cave_locations:
            self.set_action(i*32, j*32, lambda text="Sleep here?", action=self.sleep: self.show_yes_no_dialog(text, action))
        self.set_tile_action(46, 21, self.show_cave)
        self.set_tile_action(46, 20, self.hide_cave)

    def show_cave(self):
        self.cave_foreground.visible=False

    def hide_cave(self):
        self.cave_foreground.visible=True

    def end_stage(self):
        if not self.player.sleep and not self.player.food and not self.player.water:
            Interface.switch_stage("Farm")
        else:
            Interface.switch_stage("City")

    def default_action(self):
        self.show_yes_no_dialog("Sleep here?", lambda x=self.end_stage: self.fade_out(x))

    def drink_water(self):
        self.player.water=True
        self.icons['water'].image=self.icon_images['water']

    def eat_food(self):
        self.player.food=True
        self.icons['food'].image=self.icon_images['food']

    def sleep(self):
        self.player.sleep=True
        self.icons['sleep'].image=self.icon_images['sleep']
        self.fade_out(self.end_stage)

    def get_locations(layer_data, width, height):
        locations=[]
        for i in xrange(0,width):
            for j in xrange(0, height):
                if layer_data[i+j*width]:
                    locations.append((i,height-1-j))
        return locations

    def load_map(self, mapname):
        width, height=Stage.load_map(self, mapname)
        self.tree_locations=[]
        self.fruit_tree_locations=[]
        self.water_locations=[]
        self.cave_locations=[]
        for layer in self.layers:
            if layer["name"]=="Trees":
                for i in xrange(0,width):
                    for j in xrange(0, height):
                        if layer['data'][i+j*width]:
                            self.tree_locations.append((i,height-1-j))
            if layer["name"]=="Fruit Trees":
                for i in xrange(0,width):
                    for j in xrange(0, height):
                        if layer['data'][i+j*width]:
                            self.fruit_tree_locations.append((i,height-1-j))
            if layer["name"]=="Water":
                for i in xrange(0,width):
                    for j in xrange(0, height):
                        if layer['data'][i+j*width]:
                            self.water_locations.append((i,height-1-j))
            if layer["name"]=="Cave Floor":
                for i in xrange(0,width):
                    for j in xrange(0, height):
                        if layer['data'][i+j*width]:
                            self.cave_locations.append((i,height-1-j))
        return width, height

    def on_draw(self):
        Stage.on_draw(self)
        for icon in self.icons.values():
            icon.draw()
